/**********************************************************************
** Copyright (C) 2000-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qtopia Environment.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/
#include "battery.h"
#include "batterystatus.h"

#include "batterydata.h"
#include <qpe/config.h>

#if defined(Q_WS_QWS) && !defined(QT_NO_COP)
#include <qpe/qcopenvelope_qws.h>
#endif

#include <qpainter.h>
#include <qtimer.h>
#include <qapplication.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include <unistd.h>
#include <qdatetime.h>

#include <qpe/applnk.h>


BatteryMeter::BatteryMeter( QWidget *parent )
  : QWidget( parent ), charging(false)
{
  
  menu = 0;
  batt_mode = false;

  QFile f("/proc/deviceinfo/product");
  if(f.open(IO_ReadOnly) ){
    QString s;
    f.readLine(s, 9);
    if (s == QString("SL-C3000") ){
      isPXA270 = true;
    }else{
      isPXA270 = false;
    }
    f.close();
  }else{
    isPXA270 = false;
  }


  loadConfig();  

  myChannel = new QCopChannel("QPE/BatteryPlus", this);
  connect(myChannel, SIGNAL(received(const QCString&, const QByteArray&)),
	  this, SLOT(receiveMessage(const QCString&, const QByteArray&)));
  
  ps = new BatteryData;
  startTimer( 10000 );
  setFixedWidth(QMAX(AppLnk::smallIconSize()*3/4,6));
  setFixedHeight(AppLnk::smallIconSize());
  chargeTimer = new QTimer( this );
  //    connect( chargeTimer, SIGNAL(timeout()), this, SLOT(chargeTimeout()) );
  timerEvent(0);
}

BatteryMeter::~BatteryMeter()
{
  disconnect(myChannel, SIGNAL(received(const QCString&, const QByteArray&)),
	     this, SLOT(receiveMessage(const QCString&, const QByteArray&)));
  delete myChannel;
  delete (QWidget *) batteryView;
  delete ps;
}

QSize BatteryMeter::sizeHint() const
{
  return QSize(QMAX(AppLnk::smallIconSize()*3/4,6),AppLnk::smallIconSize());
}

void BatteryMeter::showPowerStatus()
{
  if ( batteryView && batteryView->isVisible() ) {
    delete (QWidget *) batteryView;
    // batteryView becomes 0 because it's a QGuardedPtr
  } else {
    batteryView = new BatteryStatus( ps, TRUE, 0, WStyle_StaysOnTop | WType_Popup | WDestructiveClose );
    batteryView->setFrameStyle( QFrame::PopupPanel | QFrame::Raised );
    bvsz = QSize();
    updateBatteryViewGeometry();
    batteryView->raise();
    batteryView->show();
    if (percent <0)
      percent=100;
    batteryView->updatePercent( percent );
  }
}

bool BatteryMeter::updateBatteryViewGeometry()
{
  QSize sz=batteryView->sizeHint();
  if ( sz != bvsz ) {
    bvsz = sz;
    QRect r(batteryView->pos(),batteryView->sizeHint());
    QPoint curPos = mapToGlobal( rect().topLeft() );
    int lp = qApp->desktop()->width() - batteryView->sizeHint().width();
    r.moveTopLeft( QPoint(lp, curPos.y()-batteryView->sizeHint().height()-1) );
    batteryView->setGeometry(r);
    return TRUE;
  }
  return false;
}

void BatteryMeter::timerEvent( QTimerEvent * )
{
  BatteryData prev = *ps;
  *ps = BatteryDataManager::readStatus();
  
  percent = ps->batteryPercentRemaining();

  
  if ( prev != *ps ){
    if ( !charging && ps->batteryStatus() == BatteryData::Charging ) {
		    charging = TRUE;
		    chargeTimer->stop();
    } else if ( charging && ps->batteryStatus() != BatteryData::Charging ) {
      charging = FALSE;
      chargeTimer->stop();
    }
    
    Clock_list* cl = clock_list;
    
    if(batt_mode){
      while(cl->name != "" ) {
	cl->checked = false;
	if (charging == TRUE && cl->batt_pc == -1){
	  changeClock(cl->clock, cl->vcore);
	  cl->checked = true;
	}else if (charging == FALSE && percent == cl->batt_pc){
	  changeClock(cl->clock, cl->vcore);
	  cl->checked = true;
	}
	cl++;
      }
    }
    
    repaint(FALSE);
    if ( batteryView ) {
      if ( updateBatteryViewGeometry() ) {
	batteryView->updatePercent( percent, FALSE );
	batteryView->update();
      } else {
	batteryView->updatePercent( percent );
      }
    }
  }

}

void BatteryMeter::chargeTimeout()
{
  //    percent += 20;
  //    if ( percent > 100 )
  //	percent = 0;
  
  //    repaint(FALSE);
  //    if ( batteryView )
  //	batteryView->updatePercent( percent );
}

void BatteryMeter::loadConfig()
{
  Clock_list* cl=clock_list;

  if(isPXA270){
    FILE *fp;
    fp=fopen("/proc/zaurus/CCCR", "r");
    if( fp ){
      fscanf(fp, "%x", &cpuregister);
      fclose(fp);
    }
    fp=fopen("/proc/zaurus/VCORE", "r");
    if( fp ){
      fscanf(fp, "%x", &vcoreregister);
      fclose(fp);
    }
    vcoreregister &= 0x0f;
  }else {
    FILE *fp;
    fp=fopen("/proc/cccr_change", "r");
    if( fp ){
      fscanf(fp, "%x", &cpuregister);
      fclose(fp);
    }
  }
  
  QFile f("/home/zaurus/Settings/BatteryPlus.conf");
  if ( f.open(IO_ReadOnly) ) {
    QTextStream t( &f );
    QString s;
    while ( !t.eof() ) {
      s = t.readLine();
      
      bool ok;
      
      QStringList menulist = QStringList::split(',', s, TRUE);
      if (menulist[0].simplifyWhiteSpace() != "") {
	if (menulist[0].simplifyWhiteSpace() == "Battery_Mode"){
	  batt_mode = true;
	}else{
	  cl->name = menulist[0].simplifyWhiteSpace();			
	  cl->clock = menulist[1].simplifyWhiteSpace().toInt(&ok,16);
	  cl->vcore = menulist[2].simplifyWhiteSpace().toInt(&ok,16);
	  cl->batt_pc = menulist[3].simplifyWhiteSpace().toInt(&ok,10);
	  cl->checked = (menulist[4].simplifyWhiteSpace() == "STD") ? TRUE : FALSE;

	  cl++;
	}
      }
    }
    f.close();
  }
  
  
  if (cl == clock_list){
    if(isPXA270){
      cl->name = C3000_SETTING1;
      cl->clock = 0x02000218;
      cl->vcore = 0x0e;
      cl++;
      
      cl->name = C3000_SETTING2;
      cl->clock = 0x02000214;
      cl->vcore = 0x0b;
      cl++;
      
      cl->name = C3000_SETTING3;
      cl->clock = 0x02000210;
      cl->vcore = 0x09;
      cl->checked = true;
      cl++;
    }else {
      cl->name = SETTING1;
      cl->clock = 0x0242;
      cl++;
      
      cl->name = SETTING2;
      cl->clock = 0x0161;
      cl++;
      
      cl->name = SETTING3;
      cl->clock = 0x0241;
      cl->checked = true;
      cl++;
    }
  }
  
  
  
}

void BatteryMeter::mousePressEvent( QMouseEvent *)
{
  
  if (!menu)
    {
      menu = new QPopupMenu(this);
      menu->setCheckable(TRUE);
    }
  else
    {
      menu->clear();
    }
  
  if(isPXA270){
    FILE *fp;
    fp=fopen("/proc/zaurus/CCCR", "r");
    if( fp ){
      fscanf(fp, "%x", &cpuregister);
      fclose(fp);
    }
    fp=fopen("/proc/zaurus/VCORE", "r");
    if( fp ){
      fscanf(fp, "%x", &vcoreregister);
      fclose(fp);
    }
    vcoreregister &= 0x0f;
  }else {
    FILE *fp;
    fp=fopen("/proc/cccr_change", "r");
    if( fp ){
      fscanf(fp, "%x", &cpuregister);
      fclose(fp);
    }
  }
  

  
  Clock_list* cl = clock_list;
  

  int id;
  menu->insertItem(STD_SETTING);
  menu->insertSeparator();
  while(cl->name != "" ){
    QString s;
    if(cl->batt_pc == 0)
      s = "";
    else if(cl->batt_pc == -1)
      s = "  AC";
    else
      s.sprintf(" %3d", cl->batt_pc);

    id = menu->insertItem(cl->name + s);
    menu->setItemChecked(id, cl->checked);
    cl++;
  }

  cl = clock_list;


  menu->insertSeparator();
  id = menu->insertItem(BATT_MODE);
  menu->setItemChecked(id, batt_mode);
  menu->insertSeparator();
  menu->insertItem(POWER_STATUS);
  
  SIG_sended=FALSE;
  connect(menu, SIGNAL(activated(int)), this, SLOT(action(int)));
  
  
  QPoint curPos = mapToGlobal( QPoint(0,0) );
  QSize sh = menu->sizeHint();
  menu->popup( curPos-QPoint((sh.width()-width())/2,sh.height()) );
  
}

void BatteryMeter::action(int id)
{
  if (!SIG_sended){
    QString menu_selected = menu->text(id);
    
    SIG_sended = TRUE;
    if (menu_selected == POWER_STATUS){
      showPowerStatus();
      return;
    }

    if (menu_selected == BATT_MODE){
      batt_mode = !batt_mode;
      return;
    }

    Clock_list* cl = clock_list;
    
    while(cl->name != "" ) {
      QString s;
      if(cl->batt_pc == 0)
	s = "";
      else if(cl->batt_pc == -1)
	s = " AC";
      else
	s.sprintf(" %3d", cl->batt_pc);


      if ( (cl->name + s) == menu_selected) {
	cl->checked = true;
	if( (cl->clock != cpuregister) || ( (cl->vcore != vcoreregister) && isPXA270) ) {
	  changeClock(cl->clock, cl->vcore);
	}
      }else {
	cl->checked = false;
      }
      cl++;
    }  

  }
}


void BatteryMeter::changeClock(int clock, int vcore)
{

  if(isPXA270){
    if( (vcoreregister == vcore) && (cpuregister == clock) )
      return;
    

    if(vcoreregister > vcore) {
      FILE *fp;
      fp=fopen("/proc/zaurus/CCCR", "w");
      if( fp ){
      fprintf(fp, "%08X\n",clock);
      fclose(fp);
      }
      fp=fopen("/proc/zaurus/VCORE", "w");
      if( fp ){
	fprintf(fp, "%02X\n",vcore);
      fclose(fp);
      }
    }else{
      FILE *fp;
      fp=fopen("/proc/zaurus/VCORE", "w");
      if( fp ){
	fprintf(fp, "%02X\n",vcore);
	fclose(fp);
      }
      fp=fopen("/proc/zaurus/CCCR", "w");
      if( fp ){
	fprintf(fp, "%08X\n",clock);
	fclose(fp);
      }
    }
    
  }else{
    if( cpuregister == clock )
      return;
    
    FILE *fp;
    fp=fopen("/proc/cccr_change", "w");
    if( fp ){
      fprintf(fp, "%08X\n",clock);
      fclose(fp);
    }
  }

  cpuregister = clock;
  vcoreregister = vcore;
  vcoreregister &= 0x0f;
  return;

}

void BatteryMeter::paintEvent( QPaintEvent* )
{
  QPainter p(this);
  
  QColor c;
  QColor darkc;
  QColor lightc;
  if ( ps->acStatus() == BatteryData::Offline ) {
    c = blue.light(120);
    darkc = c.dark(120);
    lightc = c.light(140);
  } else if ( ps->acStatus() == BatteryData::Online ) {
    percent=100;
    c = green.dark(130);
    darkc = c.dark(120);
    lightc = c.light(180);
  } else {
    c = red;
    darkc = c.dark(120);
    lightc = c.light(160);
  }
  
  //
  // To simulate a 3-d battery, we use 4 bands of colour.  From left
  // to right, these are: medium, light, medium, dark.  To avoid
  // hardcoding values for band "width", figure everything out on the run.
  //
  int	batt_width;		    // width of each band
  int	batt_height;		    // battery height (not including terminal)
  int	used_height;		    // used amount of battery (scanlines)
  
  int	batt_yoffset;		    // top of terminal
  int batt_xoffset;		    // left edge of core
  
  int	band_width;		    // width of colour band
  
  int w = QMIN(height()/2, width());
  band_width = (w-2) / 4;
  if ( band_width < 1 )
    band_width = 1;
  
  batt_width = 4 * band_width + 2;	// +2 for 1 pixel border on both sides
  batt_height = height()-2;
  batt_xoffset = (width() - batt_width) / 2;
  batt_yoffset = (height() - batt_height) / 2;
  
  //
  // Battery border.  +1 to make space for the terminal at row 0.
  //
  p.setPen(QColor(80, 80, 80));
  p.drawRect(batt_xoffset, batt_yoffset+1, batt_width, batt_height);
  
  //
  // Draw terminal.  +1 to take into account the left border.
  //
  p.drawLine(batt_xoffset + band_width + 1, batt_yoffset,
	     batt_xoffset + 3 * band_width, batt_yoffset);
  
  batt_height -= 2;	// -2 because we don't want to include border
  batt_yoffset += 2;  // +2 to account for border and terminal
  batt_xoffset++;
  
  //
  // 100 - percent, since percent is amount remaining, and we draw
  // reverse to this.
  //
  used_height = (100 - percent) * batt_height / 100;
  if (used_height < 0)
    used_height = 0;
  
  //
  // Drained section.
  //
  if (used_height != 0) {
    p.setPen(NoPen);
    p.setBrush(gray);
    p.drawRect(batt_xoffset, batt_yoffset, band_width, used_height);
    p.drawRect(batt_xoffset+2*band_width, batt_yoffset, band_width, used_height);
    
    p.setBrush(gray.light(130));
    p.drawRect(batt_xoffset+band_width, batt_yoffset, band_width, used_height);
    
    p.setBrush(gray.dark(120));
    p.drawRect(batt_xoffset+3*band_width, batt_yoffset, band_width, used_height);
  }
  
  //
  // Unused section.
  //
  if ( batt_height - used_height > 0 ) {
    int unused_offset = used_height + batt_yoffset;
    int unused_height = batt_height - used_height;
    p.setPen(NoPen);
    p.setBrush(c);
    p.drawRect(batt_xoffset, unused_offset, band_width, unused_height);
    p.drawRect(batt_xoffset+2*band_width, unused_offset, band_width, unused_height);
    
    p.setBrush(lightc);
    p.drawRect(batt_xoffset+band_width, unused_offset, band_width, unused_height);
    
    p.setBrush(darkc);
    p.drawRect(batt_xoffset+3*band_width, unused_offset, band_width, unused_height);
  }
}

void BatteryMeter::receiveMessage(
                const QCString& msg, const QByteArray& data)
{
  QDataStream stream(data, IO_ReadOnly);

  FILE *fp;

  if(msg == "changeClock(int,int)"){
    int clock;
    int vcore;
    stream >> clock >> vcore;
    changeClock(clock,vcore);

  } else if(msg == "change(QString)"){
    QString name;
    stream >> name;

    Clock_list* cl = clock_list;
    
    while(cl->name != "" ) {
      if ( cl->name == name) {
	cl->checked = true;
	if( (cl->clock != cpuregister) || ( (cl->vcore != vcoreregister) && isPXA270) ) {
	  changeClock(cl->clock, cl->vcore);
	}
      }else {
	cl->checked = false;
      }
      cl++;
    }  

  } else if(msg == "batteryMode(int)"){
    int mode;
    stream >> mode;
    batt_mode = mode;

  } else if(msg == "readConf()"){
    if ( QCopChannel::isRegistered("QPE/BatteryRecv") ){

      {
	QCopEnvelope e("QPE/BatteryRecv", "freqMenu(QString)" );
	e << QString("MenuList");
      }

      Clock_list* cl = clock_list;
      
      while(cl->name != "" ){
	QCopEnvelope e("QPE/BatteryRecv", "freqMenu(QString)" );
	e << cl->name;
	cl++;
      }

      cl = clock_list;
      while(cl->name != "" ) {
	if( (cl->clock == cpuregister) && ( (cl->vcore == vcoreregister) || !isPXA270) ) {
	  QCopEnvelope e("QPE/BatteryRecv", "currentConf(QString)" );
	  e << QString(cl->name);
	}
	cl++;
	
      }
	
    }else{

    }

  }


}
